#! /bin/sh
# New Relic agent installation script.
# (C) Copyright 2010-2011 New Relic Inc. All rights reserved.

# NOTE: this script has had comments and whitespace stripped.
: ${nrversion:="2.2.1.185"}
myuid=`id -u 2>/dev/null`
if [ "${myuid}" != "0" ]; then
echo "ERROR: $0 must be run as root" >&2
exit 1
fi
if [ -n "${NR_INSTALL_SHELL}" -a -z "${NR_KSH_EXECED}" ]; then
[ -x "${NR_INSTALL_SHELL}" ] && NR_KSH_EXECED=1 exec "${NR_INSTALL_SHELL}" -c $0 "$@"
fi
if [ -n "${BASH_VERSION}" ]; then
case $BASH_VERSION in
[3456789].* )  NR_KSH_EXECED=1 ;;
esac
fi
if [ -z "${NR_INSTALL_NOKSH}" -a -z "${NR_KSH_EXECED}" ]; then
[ -x /bin/bash ] && NR_KSH_EXECED=1 exec /bin/bash $0 "$@"
[ -x /usr/bin/bash ] && NR_KSH_EXECED=1 exec /usr/bin/bash $0 "$@"
[ -x /bin/ksh ] && NR_KSH_EXECED=1 exec /bin/ksh $0 "$@"
[ -x /usr/bin/ksh ] && NR_KSH_EXECED=1 exec /usr/bin/ksh $0 "$@"
fi
LANG=
nl='
'
if echo ' \c' | grep 'c' > /dev/null 2>&1; then
en='-n'
ec=
else
en=
ec='\c'
fi
ispkg=
[ -n "${NR_INSTALL_LOCATION}" ] && ispkg=1
imode=
if [ -n "$1" ]; then
[ "$1" = "install" ] && imode=$1
[ "$1" = "uninstall" ] && imode=$1
[ "$1" = "purge" ] && imode=$1
fi
if [ -n "${NR_INSTALL_SILENT}" ]; then
if [ -z "$imode" ]; then
echo "ERROR: must invoke with either 'install' or 'uninstall' in silent mode." >&2
exit 1
fi
fi
ostype=
if [ -f /etc/redhat-release -o -f /etc/redhat_version ]; then
ostype=rhel
elif [ -d /etc/dpkg ]; then
ostype=debian
elif [ -f /etc/release ]; then
if grep Solaris /etc/release > /dev/null 2>&1; then
ostype=solaris
fi
fi
if [ -z "${ostype}" ]; then
tus=`uname -s 2> /dev/null`
[ "x${tus}" = "xDarwin" ] && ostype=darwin
[ "x${tus}" = "xFreeBSD" ] && ostype=freebsd
fi
: ${ostype:=generic}
: ${PATH_SEPARATOR:=':'}
if [ -z "${ispkg}" ]; then
myloc=
case "$0" in
*/*) myloc=$0 ;;
*) oIFS=$IFS
IFS=$PATH_SEPARATOR
for d in $PATH; do
IFS="${oIFS}"
[ -z "$d" ] && d="."
[ -r "$d/$0" ] && {
myloc="$d/$0"
break
}
done
IFS="${oIFS}"
;;
esac
[ -z "${myloc}" ] && myloc=$0
if [ -z "${myloc}" ]; then
echo "ERROR: existential crisis: could not find myself. Re-run with absolute path." >&2
exit 1
fi
mydir=${myloc%/*}
NR_INSTALL_LOCATION=`(cd "${mydir}" 2> /dev/null; pwd 2>/dev/null)`
fi
ilibdir="${NR_INSTALL_LOCATION}"
agent=PHP
if [ -f ${NR_INSTALL_LOCATION}/setup.py ]; then
agent=Python
fi
arch=`(uname -m) 2> /dev/null` || arch="unknown"
os=`(uname -s) 2> /dev/null` || os="unknown"
case "${arch}" in
i[3456789]86 | i86pc) arch=x86 ;;
*64* | *amd*)         arch=x64 ;;
unknown)              arch=x86 ;;
esac
case "${os}" in
[dD]arwin* ) arch="universal" ;;
*) ;;
esac
if [ -f "${ilibdir}/.nrarch" ]; then
. "${ilibdir}/.nrarch"
else
darch="${arch}"
fi
[ "${NR_INSTALL_ARCH}" = "x86" -o "${NR_INSTALL_ARCH}" = "x64" -o "${NR_INSTALL_ARCH}" = "universal" ] && arch="${NR_INSTALL_ARCH}"
dmissing=
fmissing=
check_dir() {
[ -d $1 ] || dmissing="${1}${nl}${dmissing}"
}
check_file() {
[ -f $1 ] || fmissing="${1}${nl}${fmissing}"
}
xtraddir=
if [ -n "${ispkg}" ]; then
daemon=/usr/bin/newrelic-daemon
else
xtradir="${ilibdir}/daemon"
if [ "${agent}" = "Python" ]; then
daemon="${ilibdir}/daemon/newrelic-daemon"
else
daemon="${ilibdir}/daemon/newrelic-daemon.${darch}"
fi
fi
for d in "${ilibdir}/scripts" "${ilibdir}/agent" ${xtradir}; do
check_dir "${d}"
done
if [ "${agent}" = "PHP" ]; then
check_dir "${ilibdir}/agent/${arch}"
if [ "${arch}" = "x64" ]; then
check_dir "${ilibdir}/agent/x86"
fi
fi
if [ -n "${dmissing}" ]; then
echo "ERROR: the following directories could not be found:" >&2
echo "${dmissing}" | sed -e 's/^/   /' >&2
fi
if [ "${agent}" = "PHP" ]; then
NRIUTIL="${ilibdir}/scripts/newrelic-iutil.${darch}"
[ -x "${NRIUTIL}" ] || fmissing="${NRIUTIL}${nl}${fmissing}"
fi
if [ -z "${ispkg}" ]; then
check_file "${daemon}"
check_file "${ilibdir}/scripts/init.rhel"
check_file "${ilibdir}/scripts/init.debian"
check_file "${ilibdir}/scripts/init.solaris"
check_file "${ilibdir}/scripts/init.freebsd"
check_file "${ilibdir}/scripts/init.generic"
check_file "${ilibdir}/scripts/newrelic.sysconfig"
fi
if [ "${agent}" = "PHP" ]; then
check_file "${ilibdir}/scripts/newrelic.ini.template"
fi
check_file "${ilibdir}/scripts/newrelic.cfg.template"
if [ "${ostype}" = "solaris" ]; then
check_file "${ilibdir}/scripts/newrelic.xml"
fi
if [ "${agent}" = "PHP" ]; then
for pmv in "20050922" "20060613" "20090626"; do
check_file "${ilibdir}/agent/${arch}/newrelic-${pmv}.so"
check_file "${ilibdir}/agent/${arch}/newrelic-${pmv}-zts.so"
if [ "${arch}" = "x64" ]; then
check_file "${ilibdir}/agent/x86/newrelic-${pmv}.so"
check_file "${ilibdir}/agent/x86/newrelic-${pmv}-zts.so"
fi
done
fi
if [ -n "${fmissing}" ]; then
echo "ERROR: the following files could not be found:" >&2
echo "${fmissing}" | sed -e 's/^/   /' >&1
fi
if [ -n "${dmissing}" -o -n "${fmissing}" ]; then
cat <<EOF
 
The New Relic installation directory is incomplete. The files or directories
listed above could not be found. This usually means that you have a corrupt
installation archive. Please re-download your New Relic software and try
again. If the problem persists, please contact support@newrelic.com and report
the issue. Be sure to include all the above output in your bug report. We
apologise for the inconvenience.
 
EOF
exit 1
fi
cleanup() {
(cd /tmp
tar cf "${logtar}" nrinstall-${nritemp}.log nrinstall-${nritemp}.sysinfo > /dev/null 2>&1)
rm -f ${nrilog} ${nrsysinfo} > /dev/null 2>&1
}
trap cleanup EXIT
nritemp=`date +%Y%m%d-%H%M%S 2> /dev/null`
nritemp="${nritemp}-$$"
nrilog=/tmp/nrinstall-${nritemp}.log
nrsysinfo=/tmp/nrinstall-${nritemp}.sysinfo
logtar=/tmp/nrinstall-${nritemp}.tar
now=`date 2>/dev/null`
rm -f $nrilog $nrsysinfo $logtar > /dev/null 2>&1
echo "New Relic Agent Install Log dated: $now" > $nrilog
echo "New Relic Agent Install sysinfo dated: $now" > $nrsysinfo
echo "NRVERSION: ${nrversion}" >> $nrsysinfo
echo "NRVERSION: ${nrversion}" >> $nrilog
echo "NR_INSTALL_LOCATION: ${NR_INSTALL_LOCATION}" >> $nrsysinfo
echo "ARCH: ${arch}" >> $nrsysinfo
echo "DARCH: ${darch}" >> $nrsysinfo
nruname=`uname -a 2>/dev/null`
echo "UNAME: $nruname" >> $nrsysinfo
env | sort | sed -e 's/^/ENV: /' >> $nrsysinfo
if [ -z "${ispkg}" ]; then
if [ -x /usr/bin/yum ]; then
/usr/bin/yum list -q 2> /dev/null | sed -n -e '/installed$/ {s/^/YUM: /;s/[ 	]*installed$//;p}' >> $nrsysinfo
fi
if [ -x /usr/bin/dpkg-query ]; then
/usr/bin/dpkg-query -W -f 'DPKG: ${Package} ${Version} ${Architecture}\n' >> $nrsysinfo 2>/dev/null
fi
fi
error() {
echo "ERROR: $@" >> $nrilog
echo "ERROR: $@" >&2
}
fatal() {
echo "FATAL: $@" >> $nrilog
echo "FATAL: $@" >&2
cat >&2 <<EOF
FATAL: New Relic agent installation failed. Please contact support@newrelic.com
       and report the above error. We have also created a tar file with log
       files and other system info that can help solve the problem. If the file
       $logtar exists please attach it to your bug report.
       We apologise for the inconvenience.
EOF
exit 1
}
log() {
echo "$@" >> $nrilog
}
logcmd() {
echo "Executing: $@" >> $nrilog
$@ >> $nrilog 2>&1
ret=$?
echo "Command returned: $ret" >> $nrilog
return $ret
}
add_to_path() {
case "${PATH}" in
$1:*) ;;
*:$1) ;;
*:$1:*) ;;
*) [ -d "$1" ] && PATH="${PATH}${PATH_SEPARATOR}$1" ;;
esac
}
add_to_path /usr/local/bin
add_to_path /usr/local/zend/bin
add_to_path /usr/local/php-5.1/bin
add_to_path /usr/local/php-5.2/bin
add_to_path /usr/local/php-5.3/bin
add_to_path /opt/local/bin
add_to_path /usr/php/bin
add_to_path /usr/php-5.1/bin
add_to_path /usr/php-5.2/bin
add_to_path /usr/php-5.3/bin
add_to_path /usr/php/5.1/bin
add_to_path /usr/php/5.2/bin
add_to_path /usr/php/5.3/bin
add_to_path /opt/php/bin
add_to_path /opt/zend/bin
add_to_path /opt/php-5.1/bin
add_to_path /opt/php-5.2/bin
add_to_path /opt/php-5.3/bin
if [ -n "${NR_INSTALL_PATH}" ]; then
oIFS="${IFS}"
IFS=$PATH_SEPARATOR
set -- $NR_INSTALL_PATH
while [ -n "$1" ]; do
add_to_path $1
shift
done
IFS="${oIFS}"
fi
log "Final PATH=$PATH"
nrphplist=
if [ "${agent}" = "PHP" ]; then
oIFS="${IFS}"
IFS=${PATH_SEPARATOR}
set -- $PATH
while [ -n "$1" ]; do
pe=$1; shift
if [ -x "${pe}/php-config" ]; then
log "Found PHP in ${pe}"
nrphplist="${nrphplist}${PATH_SEPARATOR}${pe}"
nrphplist=${nrphplist#${PATH_SEPARATOR}}
elif [ -x "${pe}/php" ]; then
log "Found PHP in ${pe}"
nrphplist="${nrphplist}${PATH_SEPARATOR}${pe}"
nrphplist=${nrphplist#${PATH_SEPARATOR}}
else
log "No PHP found in ${pe}"
fi
done
IFS="${oIFS}"
fi
if [ -n "$NR_INSTALL_PHPLIST" ]; then
nrphplist=$NR_INSTALL_PHPLIST
fi
if [ -z "${NR_INSTALL_SILENT}" ]; then
bold=`tput bold 2> /dev/null`
rmso=`tput sgr0 2> /dev/null`
fi
banner() {
if [ -z "${NR_INSTALL_SILENT}" ]; then
clear
echo "${bold}New Relic Agent Installation (interactive mode)${rmso}"
echo "==============================================="
echo ""
fi
}
getyn() {
prompt="$1"
valid=0
while [ $valid -eq 0 ]; do
echo ${en} "${prompt} ([y]es, [n]o or x to e[x]it): ${ec}"
read choice
case "${choice}" in
y* | Y* )  return 0 ;;
n* | N* )  return 1 ;;
x* | X* | [eE][xX][iI][tT]) exit 0 ;;
*) ;;
esac
done
}
if [ "${agent}" = "PHP" ]; then
log "Final PHP list is: $nrphplist"
fi
if [ -z "${nrphplist}" ]; then
numphp=0
else
oIFS="$IFS"
IFS=${PATH_SEPARATOR}
set -- $nrphplist
numphp=$#
IFS="${oIFS}"
fi
choose_list() {
prompt="$1"; shift
upper=$1; shift
valid=0
chosen=
while [ $valid -eq 0 ]; do
echo ${en} "${prompt} (1-${upper}, 0 to exit): ${ec}"
read choice
case "${choice}" in
0 | [1-9] | [1-9][0-9] | [1-9][0-9][0-9]) ;;
*) choice= ;;
esac
if [ -n "${choice}" ]; then
if [ $choice -ge 0 -a $choice -le $upper ]; then
valid=1
fi
fi
done
chosen="${choice}"
return $choice
}
if [ -z "${imode}" ]; then
banner
if [ "${agent}" = "Python" ]; then
whatstr="Daemon"
else
whatstr="Agent and Daemon"
fi
cat <<EOF
    Please select from one of the following options:
 
    1)  Install ${bold}New Relic${rmso} ${whatstr}
    2)  Uninstall ${bold}New Relic${rmso} ${whatstr}
 
    0)  Exit
 
EOF
choose_list "   Enter choice" 2
case ${chosen} in
0)  exit 0 ;;
2)  imode="uninstall" ;;
*)  imode="install" ;;
esac
fi
log "Install mode: ${imode}"
dodaemon=
if [ "${agent}" = "PHP" ]; then
if [ "${imode}" = "install" -a -z "${nrphplist}" ]; then
log "Empty PHP directory list"
if [ -z "${NR_INSTALL_SILENT}" ]; then
cat <<EOF
 
We searched and searched and couldn't find any complete enough PHP installs.
You may well have PHP installed on your system but are lacking either the
command line version or the php-config program that should accompany each
PHP install. Sometimes, these require that you install the "dev" package
for PHP or the "cli" version. We need one or the other.
 
Please visit ${bold}http://support.newrelic.com/kb/php${rmso} and review the installation
documentation for ways in which you can customize this script to look for
PHP in non-standard locations or to do a manual install.
 
EOF
if [ -z "${ispkg}" ]; then
if getyn "   Proceed to daemon install"; then
dodaemon=yes
fi
fi
fi
if [ -z "${dodaemon}" ]; then
exit 0
fi
fi
else
dodaemon=yes
fi
get_inst() {
gin=1
gioIFS="$IFS"
IFS=${PATH_SEPARATOR}
for giod in $nrphplist; do
if [ $gin -eq $1 ]; then
echo "${giod}"
break
fi
gin=$(($gin+1))
done
IFS="${gioIFS}"
}
disp_get_php_list() {
phpulist=
if [ $numphp -eq 0 ]; then
return 0
fi
if [ $numphp -eq 1 ]; then
phpulist="${nrphplist}"
return 0
fi
if [ -n "${NR_INSTALL_SILENT}" ]; then
phpulist="${nrphplist}"
return 0
fi
pln=0
oIFS="$IFS"
IFS=${PATH_SEPARATOR}
for d in $nrphplist; do
pln=$(($pln+1))
IFS="$oIFS"
if [ $pln -lt 10 ]; then
spaces="   "
else
spaces= "  "
fi
echo "   ${pln})${spaces}${d}"
done
IFS="$oIFS"
echo ""
echo "   0)   Exit"
echo ""
valid=0
while [ $valid -eq 0 ]; do
echo ${en} "   Selection (1-${pln}, 0 to exit or all): ${ec}"
read sel
oIFS="$IFS"
IFS=", "
if [ -n "$sel" ]; then
valid=1
for e in $sel; do
case "$e" in
0) exit 0 ;;
[Aa][Ll][Ll]) phpulist="${nrphplist}" ;;
[1-9] | [1-9][0-9] | [1-9][0-9][0-9])
if [ $e -gt $pln ]; then
valid=0
else
td=`get_inst $e`
phpulist="${phpulist}:$td"
phpulist=${phpulist#:}
fi
;;
*) valid=0 ;;
esac
done
else
valid=0
fi
IFS="$oIFS"
done
echo ""
return 0
}
set_osdifile() {
osdifile=
if [ -n "${NR_INSTALL_INITFILE}" ]; then
osdifile="${NR_INSTALL_INITFILE}"
fi
if [ "${ostype}" = "darwin" ]; then
: ${osdifile:=/usr/bin/newrelic-daemon-service}
elif  [ "${ostype}" = "freebsd" ]; then
: ${osdifile:=/etc/rc.d/newrelic-daemon}
else
: ${osdifile:=/etc/init.d/newrelic-daemon}
fi
}
set_daemon_location() {
daemonloc=
if [ "${ostype}" = "solaris" ]; then
daemonloc=/opt/newrelic/bin/newrelic-daemon
else
daemonloc=/usr/bin/newrelic-daemon
fi
if [ -n "${NR_INSTALL_DAEMONPATH}" ]; then
daemonloc="${NR_INSTALL_DAEMONPATH}"
fi
}
gather_info() {
pdir="$1"
havecfg=
havebin=
pi_bin=
if [ -x "${pdir}/php-config" ]; then
havecfg=1
pi_bin=`${pdir}/php-config --php-binary 2> /dev/null`
if [ -z "${pi_bin}" -o ! -x "${pi_bin}" ]; then
if [ -x "${pdir}/php" ]; then
pi_bin="${pdir}/php"
fi
fi
fi
if [ -z "${pi_bin}" -a -x "${pdir}/php" ]; then
pi_bin="${pdir}/php"
fi
log "${pdir}: pi_bin=${pi_bin}"
phpi=
if [ -n "${pi_bin}" ]; then
havebin=1
phpi=/tmp/nrinstall-$$.phpi
rm -f ${phpi} > /dev/null 2>&1
"${pi_bin}" -i > ${phpi} 2>&1
log "${pdir}: php -i output follows {"
cat ${phpi} >> ${nrilog} 2> /dev/null
log "${pdir}: php -i output done }"
fi
log "${pdir}: havebin=${havebin} havecfg=${havecfg}"
pi_ver=
if [ -n "${havebin}" ]; then
pi_ver=`echo '<?php print phpversion(); ?>'|${pi_bin} -q 2> /dev/null`
else
pi_ver=`${pdir}/php-config --version 2> /dev/null`
fi
log "${pdir}: pi_ver=${pi_ver}"
if [ -z "${pi_ver}" ]; then
log "${pdir}: couldn't determine version"
error "could not determine the version of PHP located at:
    ${pdir}
Please consult the installation documentation to manually install New Relic
for this copy of PHP. We apologise for the inconvenience.
"
return 1
fi
case "${pi_ver}" in
5.1.* | 5.2.* | 5.3.*) ;;
*) error "unsupported version '${pi_ver}' of PHP found at:
    ${pdir}
Ignoring this particular instance of PHP.
"
log "${pdir}: unsupported version '${pi_ver}'"
return 1
;;
esac
pi_extdir=
pi_inidir_cli=
pi_inifile_cli=
pi_arch="${NR_INSTALL_ARCH}"
if [ -n "${havebin}" ]; then
pi_extdir=`echo '<?php print ini_get("extension_dir"); ?>'|${pi_bin} -q 2> /dev/null`
pi_inidir_cli=`echo '<?php print PHP_CONFIG_FILE_SCAN_DIR; ?>'|${pi_bin} -q 2> /dev/null`
pi_inifile_cli=`echo '<?php print PHP_CONFIG_FILE_PATH; ?>'|${pi_bin} -q 2> /dev/null`
if [ -z "${pi_arch}" -a "${arch}" = "x64" ]; then
if file "${pi_bin}" 2> /dev/null | grep 'ELF 32-bit' > /dev/null 2>&1; then
pi_arch="x86"
fi
fi
else
pi_extdir=`${pdir}/php-config --extension-dir 2> /dev/null`
fi
if [ -n "${pi_inidir_cli}" -a ! -d "${pi_inidir_cli}" ]; then
pi_inidir_cli=
fi
if [ -n "${pi_inifile_cli}" -a -d "${pi_inifile_cli}" ]; then
pi_inifile_cli="${pi_inifile_cli}/php.ini"
fi
if [ -z "${pi_arch}" ]; then
pi_arch="${arch}"
fi
log "${pdir}: pi_inidir_cli=${pi_inidir_cli}"
log "${pdir}: pi_inifile_cli=${pi_inifile_cli}"
pi_incdir=
if [ -n "${havecfg}" ]; then
pi_incdir=`${pdir}/php-config --include-dir 2> /dev/null`
[ -n "${pi_incdir}" -a -d "${pi_incdir}" -a -d "${pi_incdir}/main" -a -d "${pi_incdir}/Zend" ] || pi_incdir=
fi
log "${pdir}: pi_incdir=${pi_incdir}"
pi_inidir_dso=
pi_inifile_dso=
pi_extdir2=
if [ -n "${pi_incdir}" -a -f "${pi_incdir}/main/build-defs.h" ]; then
pi_inidir_dso=`sed -n '/^#define[ 	]PHP_CONFIG_FILE_SCAN_DIR / s,.*#define.*[ 	]"\(.*\)",\1,p' "${pi_incdir}/main/build-defs.h" 2> /dev/null`
pi_inifile_dso=`sed -n '/^#define[ 	]PHP_CONFIG_FILE_PATH / s,.*#define.*[ 	]"\(.*\)",\1,p' "${pi_incdir}/main/build-defs.h" 2> /dev/null`
pi_extdir2=`sed -n '/^#define[ 	]PHP_EXTENSION_DIR / s,.*#define.*[ 	]"\(.*\)",\1,p' "${pi_incdir}/main/build-defs.h" 2> /dev/null`
fi
if [ -n "${pi_inidir_dso}" -a ! -d "${pi_inidir_dso}" ]; then
pi_inidir_dso=
fi
if [ -n "${pi_inifile_dso}" -a -d "${pi_inifile_dso}" ]; then
pi_inifile_dso="${pi_inifile_dso}/php.ini"
fi
log "${pdir}: pi_inidir_dso=${pi_inidir_dso}"
log "${pdir}: pi_inifile_dso=${pi_inifile_dso}"
pi_extdir="${pi_extdir%/}"
pi_extdir2="${pi_extdir2%/}"
if [ -z "${pi_extdir}" -a -n "${pi_extdir2}" ]; then
pi_extdir="${pi_extdir2}"
pi_extdir2=
fi
if [ -z "${pi_extdir}" ]; then
error "could not determine extension directory for the PHP located at:
    ${pdir}
Please consult the installation documentation to manually install New Relic
for this copy of PHP. We apologise for the inconvenience.
"
return 1
fi
if [ ! -d "${pi_extdir}" ]; then
error "computed PHP extension directory:
    ${pi_extdir}
which is for the PHP installation located at:
    ${pdir}
does not exist. This particular instance of PHP will be skipped.
"
return 1
fi
if [ -n "${pi_extdir2}" -a "${pi_extdir}" = "${pi_extdir2}" ]; then
pi_extdir2=
fi
if [ -n "${pi_extdir2}" -a ! -d "${pi_extdir2}" ]; then
pi_extdir2=
fi
log "${pdir}: pi_extdir=${pi_extdir}"
log "${pdir}: pi_extdir2=${pi_extdir2}"
pi_modver=
case "${pi_ver}" in
5.1.*)  pi_modver="20050922" ;;
5.2.*)  pi_modver="20060613" ;;
5.3.*)  pi_modver="20090626" ;;
esac
log "${pdir}: pi_modver=${pi_modver}"
pi_zts=
if [ -n "${havebin}" ]; then
if grep 'Thread Safety' ${phpi} | grep 'enabled' > /dev/null 2>&1; then
pi_zts=yes
elif grep 'Thread Safety' ${phpi} | grep 'disabled' > /dev/null 2>&1; then
pi_zts=no
fi
fi
if [ -z "${pi_zts}" ]; then
if [ -n "${pincdir}" -a -f "${pincdir}/main/php_config.h" ]; then
if grep '^#define[ 	]*ZTS[ 	]*1' "${pincdir}/main/php_config.h" > /dev/null 2>&1; then
pi_zts="yes"
else
pi_zts="no"
fi
else
pi_zts="no"
case "${pi_extdir}" in
*non-zts* | *no-zts*) ;;
*zts*)  pi_zts="yes" ;;
*) if [ -n "${pi_extdir2}" ]; then
case "${pi_extdir2}" in
*non-zts* | *no-zts*) ;;
*zts*) pi_zts="yes" ;;
esac
fi
;;
esac
fi
fi
log "${pdir}: pi_zts=${pi_zts}"
if [ "${pi_inifile_cli}" = "${pi_inifile_dso}" ]; then
pi_inifile_dso=
fi
if [ "${pi_inidir_cli}" = "${pi_inidir_dso}" ]; then
pi_inidir_dso=
fi
if [ -n "${pi_inidir_cli}" -o -n "${pi_inidir_dso}" ]; then
if [ -n "${pi_inidir_cli}" -a -n "${pi_inidir_dso}" ]; then
sdirset=
aconfcli=`${NRIUTIL} realpath "${pi_inidir_cli}/conf.d"`
aconfdso=`${NRIUTIL} realpath "${pi_inidir_dso}/conf.d"`
if [ -n "${aconfcli}" -a -n "${aconfdso}" -a "${aconfcli}" -ef "${aconfdso}" ]; then
sdirset="${aconfcli}"
else
aconfcli=`${NRIUTIL} realpath "${pi_inidir_cli}"`
aconfdso=`${NRIUTIL} realpath "${pi_inidir_dso}"`
if [ -n "${aconfcli}" -a -n "${aconfdso}" -a "${aconfcli}" -ef "${aconfdso}" ]; then
sdirset="${aconfcli}"
fi
fi
if [ -n "${sdirset}" ]; then
pi_inifile_cli=
pi_inifile_dso=
pi_inidir_dso=
pi_inidir_cli="${sdirset}"
fi
fi
fi
if [ -n "${pi_inifile_cli}" -a -n "${pi_inidir_cli}" ]; then
pi_inifile_cli=
fi
if [ -n "${pi_inifile_dso}" -a -n "${pi_inidir_dso}" ]; then
pi_inifile_dso=
fi
if [ -n "${pi_inifile_cli}" ]; then
pi_inilist="${pi_inilist}${PATH_SEPARATOR}${pi_inifile_cli}"
pi_inilist=${pi_inilist#${PATH_SEPARATOR}}
fi
if [ -n "${pi_inifile_dso}" ]; then
pi_inilist="${pi_inilist}${PATH_SEPARATOR}${pi_inifile_dso}"
pi_inilist=${pi_inilist#${PATH_SEPARATOR}}
fi
log "${pdir}: final pi_inifile_cli=${pi_inifile_cli}"
log "${pdir}: final pi_inifile_dso=${pi_inifile_dso}"
log "${pdir}: final pi_inidir_cli=${pi_inidir_cli}"
log "${pdir}: final pi_inidir_dso=${pi_inidir_dso}"
if [ -n "${phpi}" ]; then
rm -f $phpi > /dev/null 2>&1
fi
return 0
}
install_agent_here() {
zts=
istat=
if [ "${pi_zts}" = "yes" ]; then
zts="-zts"
fi
srcf="${ilibdir}/agent/${pi_arch}/newrelic-${pi_modver}${zts}.so"
destf="${pi_extdir}/newrelic.so"
logcmd rm -f "${destf}"
if [ -z "${NR_INSTALL_USE_CP_NOT_LN}" ]; then
logcmd ln -sf "${srcf}" "${destf}" || {
istat="failed"
log "${pdir}: link '${srcf}' -> '${destf}' FAILED!"
}
else
logcmd cp -f "${srcf}" "${destf}" || {
istat="failed"
log "${pdir}: copy '${srcf}' -> '${destf}' FAILED!"
}
fi
if [ -z "${istat}" -a -n "${pi_extdir2}" ]; then
destf="${pi_extdir2}/newrelic.so"
logcmd rm -f "${destf}"
if [ -z "${NR_INSTALL_USE_CP_NOT_LN}" ]; then
logcmd ln -sf "${srcf}" "${destf}" || {
istat="failed"
log "${pdir}: link '${srcf}' -> '${destf}' FAILED!"
}
else
logcmd cp -f "${srcf}" "${destf}" || {
istat="failed"
log "${pdir}: copy '${srcf}' -> '${destf}' FAILED!"
}
fi
fi
if [ -z "${istat}" ]; then
if [ -n "${pi_inidir_cli}" -a ! -f "${pi_inidir_cli}/newrelic.ini" ]; then
if logcmd cp -f "${ilibdir}/scripts/newrelic.ini.template" "${pi_inidir_cli}/newrelic.ini"; then
logcmd chmod 644 "${pi_inidir_cli}/newrelic.ini"
if [ -z "${NR_INSTALL_SILENT}" ]; then
echo "      Install Status : ${pi_inidir_cli}/newrelic.ini created"
fi
else
istat="failed"
log "${pdir}: copy ini template to ${pi_inidir_cli}/newrelic.ini failed"
fi
fi
if [ -z "${istat}" -a -n "${pi_inidir_dso}" -a ! -f "${pi_inidir_dso}/newrelic.ini" ]; then
if logcmd cp -f "${ilibdir}/scripts/newrelic.ini.template" "${pi_inidir_dso}/newrelic.ini"; then
logcmd chmod 644 "${pi_inidir_dso}/newrelic.ini"
if [ -z "${NR_INSTALL_SILENT}" ]; then
echo "      Install Status : ${pi_inidir_dso}/newrelic.ini created"
fi
else
istat="failed"
log "${pdir}: copy ini template to ${pi_inidir_dso}/newrelic.ini failed"
fi
fi
fi
if [ -z "${istat}" ]; then
istat="OK"
fi
if [ -z "${NR_INSTALL_SILENT}" ]; then
echo "      Install Status : ${istat}"
fi
log "${pdir}: install status: ${istat}"
}
do_install() {
banner
if [ $numphp -gt 1 ]; then
if [ -z "${NR_INSTALL_SILENT}" ]; then
cat <<EOF
 
Below is a list of the directories in which we found a copy of PHP. Please
select the directory or directories for which you wish to install ${bold}New Relic${rmso}.
You can select either a single directory or multiple directories by separating
each choice with either a space or a comma. To select all of the directories
shown, please enter the special keyword 'all' (without the quotes).
 
EOF
fi
fi
disp_get_php_list
pi_inilist=
if [ -n "${phpulist}" ]; then
ioIFS="$IFS"
IFS="$PATH_SEPARATOR"
for pdir in $phpulist; do
IFS="${ioIFS}"
gather_info "${pdir}" || continue
if [ -z "${NR_INSTALL_SILENT}" ]; then
echo "Found a valid PHP in : ${pdir}"
echo "         PHP Version : ${pi_ver}"
echo "  Module API version : ${pi_modver}"
echo "    Module directory : ${pi_extdir}"
if [ -n "${pi_extdir2}" ]; then
echo "    Module directory : ${pi_extdir2}"
fi
echo "  Zend Thread Safety : ${pi_zts}"
if [ -n "${pi_inidir_cli}" ]; then
echo "   CLI ini directory : ${pi_inidir_cli}"
fi
if [ -n "${pi_inidir_dso}" ]; then
echo "   DSO ini directory : ${pi_inidir_cli}"
fi
if [ -n "${pi_inifile_cli}" ]; then
echo "        CLI ini file : ${pi_inifile_cli}"
fi
if [ -n "${pi_inifile_dso}" ]; then
echo "        DSO ini file : ${pi_inifile_dso}"
fi
fi
install_agent_here
if [ -z "${NR_INSTALL_SILENT}" ]; then
echo ""
fi
done
IFS="${ioIFS}"
fi
if [ -n "${pi_inilist}" -a -z "${NR_INSTALL_SILENT}" ]; then
cat <<EOF
 
Agent installation complete. The following php.ini files need to be modified by
hand to load the ${bold}New Relic${rmso} extension, enable it, and configure it. A sample
ini file fragment can be found at:
 
    ${ilibdir}/scripts/newrelic.ini.template
 
Without performing this step the agent will not be loaded correctly. The ini
file(s) you need to modify are:
EOF
ioIFS="${IFS}"
IFS=${PATH_SEPARATOR}
for ifile in ${pi_inilist}; do
IFS="${ioIFS}"
echo "    ${ifile}"
done
IFS="${ioIFS}"
echo ""
fi
set_daemon_location
idaemon="${dodaemon}"
if [ -z "${ispkg}" -a -z "${idaemon}" ]; then
if [ -z "${NR_INSTALL_SILENT}" ]; then
cat <<EOF
The agents need the ${bold}New Relic${rmso} proxy daemon to be installed and running. Without
this daemon no data will be transmitted to the ${bold}New Relic${rmso} web site. It is highly
recommended that you allow this script to install the daemon into the default
location, which is:
 
    ${daemonloc}
 
EOF
if [ -x "${daemonloc}" ]; then
cat <<EOF
A file by that name already exists, and should you choose to continue, it will
be replaced by this script.
 
EOF
fi
if getyn "   Install daemon"; then
idaemon=yes
else
idaemon=no
fi
else
idaemon=yes
fi
fi
if [ "x${idaemon}" = "xyes" ]; then
if [ "${ostype}" = "solaris" ]; then
logcmd mkdir -p -m 0755 /opt/newrelic/bin
fi
if logcmd rm -f "${daemonloc}"; then
if logcmd cp -f "${daemon}" "${daemonloc}"; then
logcmd chmod 750 "${daemonloc}"
log "daemon installed"
else
fatal "failed to copy new daemon into place"
fi
else
fatal "could not remove existing daemon"
fi
if [ ! -d /var/log/newrelic ]; then
if [ -d /var/log ]; then
logcmd mkdir /var/log/newrelic
logcmd chown root /var/log/newrelic
logcmd chmod 733 /var/log/newrelic
log "created /var/log/newrelic with restrictive permissions"
fi
fi
fi
if [ ! -d /etc/newrelic ]; then
if logcmd mkdir /etc/newrelic; then
logcmd chown root /etc/newrelic
logcmd chmod 700 /etc/newrelic
else
fatal "failed to create directory /etc/newrelic"
fi
fi
if [ ! -f /etc/newrelic/newrelic.cfg ]; then
nrkey="${NR_INSTALL_KEY}"
if [ -z "${NR_INSTALL_SILENT}" -a -z "${NR_INSTALL_KEY}" ]; then
cat <<EOF
 
The ${bold}New Relic${rmso} proxy daemon needs to be configured. The primary configuration
item is your ${bold}New Relic${rmso} license key. When prompted below, please enter your
license key. This will be installed, along with default configuration options,
into the file:
 
    /etc/newrelic/newrelic.cfg
 
If you do not have your license key handy you can edit this file later. Until
such time as the key has been correctly installed in this file you will not be
able to send data to the ${bold}New Relic${rmso} web site.
 
EOF
echo ${en} "   Enter New Relic license key (or leave blank): ${ec}"
read nrkey
fi
if [ -n "${nrkey}" ]; then
if sed -e "s/REPLACE_WITH_REAL_KEY/${nrkey}/" "${ilibdir}/scripts/newrelic.cfg.template" > /etc/newrelic/newrelic.cfg; then
logcmd chown root /etc/newrelic/newrelic.cfg
logcmd chmod 600 /etc/newrelic/newrelic.cfg
log "installed edited newrelic.cfg"
else
fatal "failed to install template daemon configuration file"
fi
else
if [ -z "${NR_INSTALL_SILENT}" ]; then
if logcmd cp -f "${ilibdir}/scripts/newrelic.cfg.template" /etc/newrelic/newrelic.cfg; then
logcmd chown root /etc/newrelic/newrelic.cfg
logcmd chmod 600 /etc/newrelic/newrelic.cfg
log "installed newrelic.cfg template"
else
fatal "failed to copy template daemon configuration file"
fi
fi
fi
fi
badlicense=0
if [ -f /etc/newrelic/newrelic.cfg ]; then
if grep 'REPLACE_WITH_REAL_KEY' /etc/newrelic/newrelic.cfg > /dev/null 2>&1; then
badlicense=1
log "/etc/newrelic/newrelic.cfg does not contain a valid license"
if [ -z "${NR_INSTALL_SILENT}" ]; then
cat <<EOF
 
Please be sure to edit /etc/newrelic/newrelic.cfg and add your license key
before you attempt to start the New Relic proxy daemon.
 
EOF
fi
fi
fi
set_osdifile
if [ -z "${ispkg}" ]; then
if [ ! -f "${osdifile}" ]; then
if logcmd cp -f "${ilibdir}/scripts/init.${ostype}" "${osdifile}"; then
logcmd chmod 755 "${osdifile}" || {
fatal "failed to set permissions on ${osdifile}"
}
else
fatal "failed to copy daemon init script to ${osdifile}"
fi
fi
sysconf=
if [ "${ostype}" = "rhel" ]; then
sysconf=/etc/sysconfig/newrelic-daemon
fi
if [ "${ostype}" = "debian" ]; then
sysconf=/etc/default/newrelic-daemon
fi
if [ -n "${sysconf}" -a ! -f "${sysconf}" ]; then
if logcmd cp -f "${ilibdir}/scripts/newrelic.sysconfig" "${sysconf}"; then
logcmd chmod 755 "${sysconf}" || {
fatal "failed to set permissions on ${sysconf}"
}
else
fatal "failed to copy script to ${sysconf}"
fi
fi
if [ "${ostype}" = "solaris" ]; then
logcmd /usr/sbin/svccfg import "${ilibdir}/scripts/newrelic.xml" || {
error "failed to install the New Relic daemon service"
}
fi
fi
if [ -x "${osdifile}" -a "x${badlicense}" = "x0" ]; then
NR_SILENT=yes SILENT=yes "${osdifile}" restart > /dev/null 2>&1
fi
if [ -z "${NR_INSTALL_SILENT}" ]; then
if [ -n "${dodaemon}" ]; then
if [ "${agent}" = "Python" ]; then
cat <<EOF
 
The New Relic Proxy Daemon is installed, but the agent is not. Please read
the instructions for installation of the Python agent components in the
${bold}INSTALL${rmso} file located in the same directory as this installer.
 
EOF
else
cat <<EOF
 
The New Relic Proxy Daemon is installed, but the agent is not. Please point
your favorite web browser at ${bold}http://support.newrelic.com/kb/php${rmso} for how to
install the agent by hand.
 
EOF
fi
else
cat <<EOF
 
${bold}New Relic${rmso} is now installed on your system. Congratulations!
 
The last step before you will start seeing data in your account is to restart
your web server or servers, or whatever environment hosts the PHP you wish to
monitor. For example, you may need to restart Apache or nginx, or if you are
using php-fpm and FastCGI, you will need to restart php-fpm. If you installed
${bold}New Relic${rmso} into multiple locations you may have multiple PHP hosts to restart.
If everything is working properly you should see the following two log files:
 
    /var/log/newrelic/newrelic-daemon.log
    /var/log/newrelic/php_agent.log
 
Now you should create a simple test php script that contains the line:
 
   <?php phpinfo(); ?>
 
and point your browser at the test script, and you should see the ${bold}New Relic${rmso}
extension and its settings. If you still do not see those log files or data in
your account please contact our support staff at ${bold}support@newrelic.com${rmso} and be
sure to include the following file in your bug report:
 
    ${logtar}
 
Thank you for choosing New Relic!
EOF
fi
fi
return 0
}
purge=0
remove_if_unchanged() {
target="$1"
source="$2"
dorm=1
if [ -f "${target}" -a -f "${source}" ]; then
cmp "${source}" "${target}" > /dev/null 2>&1 || dorm=0
else
dorm=0
fi
if [ "x${dorm}" = "x1" -o "x${purge}" = "x1" ]; then
logcmd rm -f "${target}"
fi
}
remove_from_here() {
destf="${pi_extdir}/newrelic.so"
if [ -f "${destf}" ]; then
logcmd rm -f "${destf}"
fi
if [ -n "${pi_extdir2}" ]; then
destf="${pi_extdir2}/newrelic.so"
if [ -f "${destf}" ]; then
logcmd rm -f "${destf}"
fi
fi
if [ -n "${pi_inidir_cli}" -a -f "${pi_inidir_cli}/newrelic.ini" ]; then
remove_if_unchanged "${pi_inidir_cli}/newrelic.ini" "${ilibdir}/scripts/newrelic.ini.template"
fi
if [ -n "${pi_inidir_dso}" -a -f "${pi_inidir_dso}/newrelic.ini" ]; then
remove_if_unchanged "${pi_inidir_dso}/newrelic.ini" "${ilibdir}/scripts/newrelic.ini.template"
fi
}
do_uninstall() {
banner
phpulist="${nrphplist}"
pi_inilist=
if [ -n "${phpulist}" ]; then
ioIFS="$IFS"
IFS=${PATH_SEPARATOR}
for pdir in $phpulist; do
IFS="${ioIFS}"
gather_info "${pdir}" || continue
if [ -z "${NR_INSTALL_SILENT}" ]; then
echo "Removing from this PHP : ${pdir}"
echo "      Module directory : ${pi_extdir}"
if [ -n "${pi_extdir2}" ]; then
echo "      Module directory : ${pi_extdir2}"
fi
fi
remove_from_here
if [ -z "${NR_INSTALL_SILENT}" ]; then
echo ""
fi
done
IFS="${ioIFS}"
fi
set_osdifile
if [ -x "${osdifile}" ]; then
NR_SILENT=yes SILENT=yes "${osdifile}" stop > /dev/null 2>&1
fi
if [ -z "${ispkg}" ]; then
remove_if_unchanged "${osdifile}" "${ilibdir}/scripts/init.${ostype}"
fi
remove_if_unchanged /etc/newrelic/newrelic.cfg "${ilibdir}/scripts/newrelic.cfg.template"
if [ -z "${ispkg}" ]; then
set_daemon_location
logcmd rm -f "${daemonloc}"
fi
if [ -z "${NR_INSTALL_SILENT}" -a "x${purge}" = "x0" ]; then
cat <<EOF
 
Removal of ${bold}New Relic${rmso} complete. If you are removing in order to perform an
upgrade, run the installation script from the new distribution. All existing
configuration files that were modified have been left in place. If you wish
to permanently remove ${bold}New Relic${rmso}, you may wish to invoke this script with the
'purge' option to completely remove ${bold}New Relic${rmso}. If so, please take a moment to
mail ${bold}support@newrelic.com${rmso} telling us why you are leaving us and what we can do
to help you or improve your experience. Thank you in advance.
 
EOF
fi
}
do_purge() {
purge=1
do_uninstall
if [ -d /etc/newrelic ]; then
logcmd rmdir /etc/newrelic
fi
if [ -d /var/log/newrelic ]; then
logcmd rm -fr /var/log/newrelic
fi
if [ -z "${NR_INSTALL_SILENT}" ]; then
cat <<EOF
 
We are sorry to see you go! If there is anything we can do to improve your
experience with the product, please mail ${bold}support@newrelic.com${rmso} and we will do
our very best to help you or to improve the product to meet your expectations.
Thank you in advance. -- The ${bold}New Relic${rmso} ${agent} Team
 
EOF
fi
return 0
}
[ "${imode}" = "install" ] && do_install
[ "${imode}" = "uninstall" ] && do_uninstall
[ "${imode}" = "purge" ] && do_purge
exit 0
